@@ -1,10 +1,11 @@
 /*
 flasm, command line assembler & disassembler of flash actionscript bytecode
-Copyright (c) 2001 Opaque Industries, (c) 2002-2005 Igor Kogan, (c) 2005 Wang Zhen
+Copyright (c) 2001 Opaque Industries, (c) 2002-2007 Igor Kogan, (c) 2005 Wang Zhen
 All rights reserved. See LICENSE.TXT for terms of use.
 */
 
 #include <stdio.h>
+#include <stdint.h>
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
@@ -578,7 +579,10 @@ static void writeTagHeader(unsigned int type, unsigned long int length)
 	if (length >= 63
 		|| type == TAG_DEFINEBITSLOSSLESS
 		|| type == TAG_DEFINEBITSLOSSLESS2
-		|| type == TAG_SOUNDSTREAMBLOCK) {
+		|| type == TAG_SOUNDSTREAMBLOCK
+		|| type == TAG_DEFINEBITS
+		|| type == TAG_DEFINEBITSJPEG2
+		|| type == TAG_DEFINEBITSJPEG3) {
 		/*
 		   long length, and also workaround for a really strange bug in flash player:
 		   the above tags must always have long length to work
@@ -842,7 +846,7 @@ void writePlaceMCStart(unsigned int clipID)
 	MCAllEventsPos = ftell(tempFile);
 
 	/* skip old events */
-	if (swfVersion == 5) {
+	if (swfVersion <= 5) {
 		/* dup logical or of all events - temporary */
 		dupWord();
 		while (getWord(updateFile) > 0) {
@@ -869,8 +873,9 @@ void writePlaceMCEnd(unsigned long int flags)
 
 	updateTmpLength(placeMCLengthPos, ftell(tempFile) - placeMCLengthPos - 4);
 
-	if (swfVersion == 5)
+	if (swfVersion <= 5) {
 		updateShort(MCAllEventsPos, (unsigned int) flags);
+	}
 	else if (swfVersion >= 6) {
 		updateShort(MCAllEventsPos, (unsigned int) (flags & 0xFFFF));
 		updateShort(MCAllEventsPos + 2, (unsigned int) ((flags >> 16) & 0xFFFF));
@@ -1402,7 +1407,7 @@ static void usage(void)
 	// printf(" (zlib %s)", ZLIB_VERSION);
 	puts("");
 	puts("");
-	puts("(c) 2001 Opaque Industries, (c) 2002-2005 Igor Kogan, (c) 2005 Wang Zhen");
+	puts("(c) 2001 Opaque Industries, (c) 2002-2007 Igor Kogan, (c) 2005 Wang Zhen");
 	puts("All rights reserved. See LICENSE.TXT for terms of use.");
 	puts("");
 	puts("Usage: flasm [command] filename");
@@ -1564,7 +1569,7 @@ static char *executablePath(void) {
 	if(GetModuleFileName(NULL, path, MAX_PATH_LEN - 1))
 		return mstrdup(path);
 #elif defined(__APPLE__)
-	unsigned long pathlen = MAX_PATH_LEN - 1;
+	uint32_t pathlen = MAX_PATH_LEN - 1;
 	if (_NSGetExecutablePath(path, &pathlen) == 0)
 		return mstrdup(path);
 #else
