@@ -1,6 +1,6 @@
 /*
 flasm, command line assembler & disassembler of flash actionscript bytecode
-Copyright (c) 2001 Opaque Industries, (c) 2002-2005 Igor Kogan, (c) 2005 Wang Zhen
+Copyright (c) 2001 Opaque Industries, (c) 2002-2007 Igor Kogan, (c) 2005 Wang Zhen
 All rights reserved. See LICENSE.TXT for terms of use.
 */
 
@@ -246,7 +246,7 @@ static unsigned int bitBuf;
 static void InitBits(FILE *f)
 {
 	bitPos = 8;
-	bitBuf = (unsigned int) fgetc(f);
+	bitBuf = ((unsigned int) fgetc(f)) & 0xff;
 }
 
 static unsigned int getBits(FILE *f, unsigned int n)
@@ -256,7 +256,7 @@ static unsigned int getBits(FILE *f, unsigned int n)
 	while (n > bitPos) {
 		n -= bitPos;
 		v |= bitBuf << n;
-		bitBuf = (unsigned int) fgetc(f);
+		bitBuf = ((unsigned int) fgetc(f)) & 0xff;
 		bitPos = 8;
 	}
 
@@ -264,7 +264,7 @@ static unsigned int getBits(FILE *f, unsigned int n)
 	v |= bitBuf >> bitPos;
 	bitBuf &= 0xff >> (8 - bitPos);
 	/* never need more than 16 bits */
-	return (unsigned int) v;
+	return (unsigned int) v & 0xffff;
 }
 
 static void printFrameNum(unsigned int frameNum)
@@ -1638,9 +1638,9 @@ static void skipColorTransform(FILE *f)
 
 static void skipFilters(FILE *f, byte numfilters)
 {
-	while (numfilters--) {
+	while(numfilters--) {
 		int filter = fgetc(f);
-		switch (filter){
+		switch(filter){
 			case FILTER_DROPSHADOW:
 				skipProtected(f, 23);
 				break;
@@ -1852,7 +1852,7 @@ static void parseButton2(FILE *f, unsigned long int length)
 	fgetc(f);
 }
 
-static int checkEvent(unsigned long int *eventptr, unsigned long int e, char *delimptr, char *str)
+static int printBitMasked(unsigned long int *eventptr, unsigned long int e, char *delimptr, char *str)
 {
 	if (*eventptr & e) {
 		*eventptr -= e;
@@ -1874,26 +1874,26 @@ static void parseEvent(FILE *f, unsigned long int event, unsigned long int lengt
 	++indent;
 	print("onClipEvent");
 
-	checkEvent(&event2, eventLoad,				&delim, "load");
-	checkEvent(&event2, eventEnterFrame,		&delim, "enterFrame");
-	checkEvent(&event2, eventUnload,			&delim, "unload");
-	checkEvent(&event2, eventMouseMove,			&delim, "mouseMove");
-	checkEvent(&event2, eventMouseDown,			&delim, "mouseDown");
-	checkEvent(&event2, eventMouseUp,			&delim, "mouseUp");
-	checkEvent(&event2, eventKeyDown,			&delim, "keyDown");
-	checkEvent(&event2, eventKeyUp,				&delim, "keyUp");
-	checkEvent(&event2, eventData,				&delim, "data");
-	checkEvent(&event2, eventInitialize,		&delim, "initialize");
-	checkEvent(&event2, eventConstruct,			&delim, "construct");
-	checkEvent(&event2, eventPress,				&delim, "press");
-	checkEvent(&event2, eventRelease,			&delim, "release");
-	checkEvent(&event2, eventReleaseOutside,	&delim, "releaseOutside");
-	checkEvent(&event2, eventRollOver,			&delim, "rollOver");
-	checkEvent(&event2, eventRollOut,			&delim, "rollOut");
-	checkEvent(&event2, eventDragOver,			&delim, "dragOver");
-	checkEvent(&event2, eventDragOut,			&delim, "dragOut");
+	printBitMasked(&event2, eventLoad,				&delim, "load");
+	printBitMasked(&event2, eventEnterFrame,		&delim, "enterFrame");
+	printBitMasked(&event2, eventUnload,			&delim, "unload");
+	printBitMasked(&event2, eventMouseMove,			&delim, "mouseMove");
+	printBitMasked(&event2, eventMouseDown,			&delim, "mouseDown");
+	printBitMasked(&event2, eventMouseUp,			&delim, "mouseUp");
+	printBitMasked(&event2, eventKeyDown,			&delim, "keyDown");
+	printBitMasked(&event2, eventKeyUp,				&delim, "keyUp");
+	printBitMasked(&event2, eventData,				&delim, "data");
+	printBitMasked(&event2, eventInitialize,		&delim, "initialize");
+	printBitMasked(&event2, eventConstruct,			&delim, "construct");
+	printBitMasked(&event2, eventPress,				&delim, "press");
+	printBitMasked(&event2, eventRelease,			&delim, "release");
+	printBitMasked(&event2, eventReleaseOutside,	&delim, "releaseOutside");
+	printBitMasked(&event2, eventRollOver,			&delim, "rollOver");
+	printBitMasked(&event2, eventRollOut,			&delim, "rollOut");
+	printBitMasked(&event2, eventDragOver,			&delim, "dragOver");
+	printBitMasked(&event2, eventDragOut,			&delim, "dragOut");
 
-	if (checkEvent(&event2, eventKeyPress, &delim, "")) {
+	if (printBitMasked(&event2, eventKeyPress, &delim, "")) {
 		key = (byte) fgetc(f);
 		parseKeyPressEvent(key);
 		length--;
@@ -1910,13 +1910,18 @@ static void parseEvent(FILE *f, unsigned long int event, unsigned long int lengt
 	--indent;
 }
 
-static void parsePlaceObject2(FILE *f, unsigned long int length)
+static void parsePlaceObject(FILE *f, unsigned long int length, unsigned int type)
 {
-	int i, flags;
-	unsigned int clipID = 0, depth;
+	int i;
+	unsigned int flags;
+	unsigned int clipID = 0;
+	unsigned int depth;
 	unsigned long int curEvent;
-
-	flags = fgetc(f);
+	
+	if (type == TAG_PLACEOBJECT2)
+		flags = fgetc(f);
+	else
+		flags = getWord(f);
 
 	if (flags & PF_ONCLIPEVENTS) {
 		printf("\n");
@@ -1925,8 +1930,9 @@ static void parsePlaceObject2(FILE *f, unsigned long int length)
 		depth = getWord(f);
 
 		/* clipID should always be present */
-		if (flags & PF_CHARACTER)
+		if (flags & PF_CHARACTER) {
 			print("placeMovieClip %u ", clipID = getWord(f));
+		}
 		else {
 			print("placeMovieClip ??? ");
 			tellUser(0, "placeMovieClip: clip ID not found");
@@ -1950,92 +1956,35 @@ static void parsePlaceObject2(FILE *f, unsigned long int length)
 		if (flags & PF_DEFINECLIP)
 			getWord(f);
 
+		if (type == TAG_PLACEOBJECT3) {
+			if (flags & PF_FILTERS)
+				skipFilters(f, (byte) fgetc(f));
+			if (flags & PF_BLENDMODE)
+				fgetc(f);
+			if (flags & PF_BITMAPCACHING)
+				fgetc(f);
+		}
+
 		printf("\n");
 
+
 		/* reserved: always 0 */
 		getWord(f);
 
-		if (swfVersion <= 5) {
-			/* all events */
-			getWord(f);
-			while ((curEvent = (unsigned long int) getWord(f)) != 0)
-				parseEvent(f, curEvent, getDoubleWord(f));
-		}
-		else if (swfVersion >= 6) {
-			/* flash 6 now supports button events for mcs, therefore going long here */
+		if (swfVersion >= 6 || type == TAG_PLACEOBJECT3) {
+			/* flash 6 supports button events for mcs, therefore going long here */
 			getDoubleWord(f);
 
 			while ((curEvent = getDoubleWord(f)) != 0)
 				parseEvent(f, curEvent, getDoubleWord(f));
 		}
-
-		if (flags & PF_CHARACTER)
-			print("end // of placeMovieClip %u\n", clipID);
-		else
-			print("end // of placeMovieClip ???\n");
-		
-	}
-	else {
-		/* no events found, skip the rest of placeObject2 */
-		skipProtected(f, length - 1);
-	}
-}
-
-static void parsePlaceObject3(FILE *f, unsigned long int length)
-{
-	int i;
-	unsigned int clipID = 0, depth, flags;
-	unsigned long int curEvent;
-
-	flags = getWord(f);
-	
-	if (flags & PF_ONCLIPEVENTS) {
-		printf("\n");
-
-		/* character depth */
-		depth = getWord(f);
-
-		/* clipID should always be present */
-		if (flags & PF_CHARACTER)
-			print("placeMovieClip %u ", clipID = getWord(f));
 		else {
-			print("placeMovieClip ??? ");
-			tellUser(0, "placeMovieClip: clip ID not found");
-		}
-
-		if (flags & PF_MATRIX)
-			skipMatrix(f);
-		if (flags & PF_COLORTRANSFORM)
-			skipColorTransform(f);
-		if (flags & PF_RATIO)
+			/* all events */
 			getWord(f);
-
-		if (flags & PF_NAME) {
-			printf("as ");
-			putchar('\'');
-			while ((i = fgetc(f)) != 0)
-				putchar((char) i);
-			putchar('\'');
+			while ((curEvent = (unsigned long int) getWord(f)) != 0)
+				parseEvent(f, curEvent, getDoubleWord(f));
 		}
 
-		if (flags & PF_DEFINECLIP)
-			getWord(f);
-		if (flags & PF_FILTERS)
-			skipFilters(f, (byte) fgetc(f));
-		if (flags & PF_BLENDMODE)
-			fgetc(f);
-		if (flags & PF_BITMAPCACHING)
-			fgetc(f);
-		
-		printf("\n");
-
-		/* reserved: always 0 */
-		getWord(f);
-		/* all events */
-		getDoubleWord(f);
-		while ((curEvent = getDoubleWord(f)) != 0)
-			parseEvent(f, curEvent, getDoubleWord(f));
-
 		if (flags & PF_CHARACTER)
 			print("end // of placeMovieClip %u\n", clipID);
 		else
@@ -2043,8 +1992,11 @@ static void parsePlaceObject3(FILE *f, unsigned long int length)
 		
 	}
 	else {
-		/* no events found, skip the rest of placeObject3 */
-		skipProtected(f, length - 2);
+		/* no events found, skip the rest of placeObject2/3 */
+		if (type == TAG_PLACEOBJECT3)
+			skipProtected(f, length - 2);
+		else
+			skipProtected(f, length - 1);
 	}
 }
 
@@ -2077,12 +2029,12 @@ static void parseMovieClip(FILE *f)
 
 			case TAG_PLACEOBJECT2:
 				/* possibly onClipEvents inside */
-				parsePlaceObject2(f, length);
+				parsePlaceObject(f, length, TAG_PLACEOBJECT2);
 				break;
 
 			case TAG_PLACEOBJECT3:
 				/* possibly onClipEvents inside */
-				parsePlaceObject3(f, length);
+				parsePlaceObject(f, length, TAG_PLACEOBJECT3);
 				break;
 
 			case TAG_SHOWFRAME:
@@ -2166,12 +2118,12 @@ void disassembleSWF(FILE *f, char *fname)
 
 			case TAG_PLACEOBJECT2:
 				/* possibly onClipEvents inside */
-				parsePlaceObject2(f, length);
+				parsePlaceObject(f, length, TAG_PLACEOBJECT2);
 				break;
 
 			case TAG_PLACEOBJECT3:
 				/* possibly onClipEvents inside */
-				parsePlaceObject3(f, length);
+				parsePlaceObject(f, length, TAG_PLACEOBJECT3);
 				break;
 
 			case TAG_DEFINEBUTTON2:
@@ -2318,16 +2270,11 @@ void disassembleSWF(FILE *f, char *fname)
 					print("\n");
 					print("fileAttributes");
 				}
-				if (attrs & ATTR_USENETWORK) {
-					printf("%cuseNetwork", delim);
-					delim = ',';
-					attrs -= ATTR_USENETWORK;
-				}
-				if (attrs & ATTR_HASMETADATA) {
-					printf("%chasMetadata", delim);
-					delim = ',';
-					attrs -= ATTR_HASMETADATA;
-				}				
+				printBitMasked(&attrs, ATTR_USENETWORK,					&delim, "attrUseNetwork");
+				printBitMasked(&attrs, ATTR_RELATIVEURLS,				&delim, "attrRelativeURLs");
+				printBitMasked(&attrs, ATTR_SUPPRESSCROSSDOMAINCACHE,	&delim, "attrSuppressCrossDomainCache");
+				printBitMasked(&attrs, ATTR_ACTIONSCRIPT3,				&delim, "attrActionScript3");
+				printBitMasked(&attrs, ATTR_HASMETADATA,				&delim, "attrHasMetadata");
 				if (attrs != 0) {
 					printf("%c%lu // unknown attribute", delim, attrs);
 					tellUser(0, "Unknown file attribute: %lu", attrs);
